% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		Axiom Internal Representation											% %
% %																				% %
% %		Each axiom is stored using the Prolog (dynamic) predicate 				% %
% % 				"tptpAxiom/8":												% %
% %																				% %
% %		tptpAxiom(File,FirstLine,LastLine,Name,Role,AxiomKey,Type,FOFormula)	% %
% %																				% %
% %			File: name of file of the axiom										% %
% %																				% %
% %			FirstLine: First line of the axiom in the file						% %
% %																				% %
% %			LastLine: Last line of the axiom in the file						% %
% %																				% %
% %			Name: name of the axiom in the tstp file 							% %
% %																				% %
% %			Role: role of the axiom												% %
% %																				% %
% %					axiom														% %
% %					conjecture													% %
% %					(text)														% %
% %																				% %
% %			AxiomKey: an atom starting fromt the name of the file				% %
% %							and followed by a number							% %
% %						comments and empty lines have the key "text"			% %
% %																				% %
% %			Type:																% %
% %																				% %
% %					unitClause													% %
% %					nonAtomicAxiom												% %
% %																				% %
% %			FO formulas are represented using the following functions:			% %
% %																				% %
% %					Exists (typed) ->		exists(LV,F)						% %
% %					For all (typed) ->		forall(LV,F)						% %
% %					Conjunction ->			and(LF)								% %
% %					Disjunction ->			or(LF)								% %
% %					Implication ->			implies(F1,F2)						% %
% %					Equivalence ->			equiv(F1,F2)						% %
% %					Negation ->				not(F)								% %
% %					Equality ->				equal(T1,T2)						% %
% %																				% %
% %				where	LV is a list of Variables								% %
% %						F, F1, F2 are formulas									% %
% %						LF is a list of formulas								% %
% %						T1, T2 are terms										% %
% %						RV is a Row variable									% %
% %						H is the head of a Row variable (a Variable)			% %
% %						S is the tail of a Row variable							% %
% %							(a Row variable)									% %
% %																				% %
% %			Terms are represented as Prolog terms								% %
% %																				% %
% %			Variables are represented using "'#var'/1"							% %
% %																				% %
% %																				% %
% %		In addition, the Prolog (dynamic) predicate "numberOfLines/1" stores	% %
% %			the number of lines in each file and "numberOfAxioms/2" stores		% %
% %			the number of axioms in each file									% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- dynamic	tptpAxiom/8, tptpGoal/5,
			numberOfTPTPLines/2, numberOfTPTPAxioms/2,
			numberOfTPTPGoals/2,numberOfTPTPTests/2,
			tptpGoalInformation/3, tptpTestInformation/5.


deletePreviousTPTPInformation:-
		retractall((tptpAxiom(_,_,_,_,_,_,_,_))),								% Delete previous information
		retractall((tptpGoal(_,_,_,_,_))),
		retractall((numberOfTPTPLines(_,_))),
		retractall((numberOfTPTPAxioms(_,_))),
		retractall((numberOfTPTPGoals(_,_))),
		retractall((numberOfTPTPTests(_,_))),
		retractall((tptpGoalInformation(_,_,_))),
		retractall((tptpTestInformation(_,_,_,_,_))).


%%% saveTPTPAxiom(in FILE, in FL, in LL, in N, in R, in K, in T, in F)
%%%		FILE: axiom file
%%%		FL: first line of the axiom in FILE
%%%		LL: last line of the axiom in FILE
%%%		N: axiom name
%%%		R: axiom role
%%%		K: axiom key
%%%		T: axiom type
%%%		F: axiom formula
%%%		Action: the axiom is stored

saveTPTPAxiom(F,FL,LL,N,R,K,T,NF):-
		(
		 (
		  retract((tptpAxiom(F,FL,LL,N,_,K,_,_))),
		  !
		 );
		 (
		  true
		 )
		),
		assertz((tptpAxiom(F,FL,LL,N,R,K,T,NF))).

%%% getTPTPAxiom(in K, out T, out F)
%%%		K: axiom key
%%%		T: axiom type
%%%		F: axiom formula

getTPTPAxiom(K,T,F):-
		tptpAxiom(_,_,_,_,_,K,T,F).

%%% getCompleteTPTPAxiom(in K, out FILE, out FL, out LL, out N, out r, out K, out T, out F)
%%%		K: axiom key
%%%		FILE: axiom file
%%%		FL: first line of the axiom in FILE
%%%		LL: last line of the axiom in FILE
%%%		N: axiom name
%%%		R: axiom role
%%%		T: axiom type
%%%		F: axiom formula

getCompleteTPTPAxiom(K,F,FL,LL,N,R,T,NF):-
		tptpAxiom(F,FL,LL,N,R,K,T,NF).

%%% getTPTPAxiomByFileLine(in FILE, in FL, out LL, out F)
%%%		FILE: axiom file
%%%		FL: first line of the axiom in FILE
%%%		C: axiom type
%%%		LL: last line of the axiom in FILE
%%%		K: axiom key
%%%		F: axiom formula

getTPTPAxiomByFileLine(F,FL,LL,N,R,K,T,NF):-
		tptpAxiom(F,FL,LL,N,R,K,T,NF).

%%% setTPTPAxiom(in K, in F)
%%%		K: axiom key
%%%		F: axiom formula
%%%		Action: the axiom of key K is update with F

setTPTPAxiom(K,NF):-
		tptpAxiom(FILE,FL,LL,N,R,K,T,_),
		retract((tptpAxiom(_,_,_,_,_,K,_,_))),
		assertz((tptpAxiom(FILE,FL,LL,N,R,K,T,NF))).

%%% getNumberOfTPTPAxioms(in F, out N)
%%%		F: file
%%%		N: number of axioms in F

getNumberOfTPTPAxioms(F,N):-
		numberOfTPTPAxioms(F,N).

%%% setNumberOfTPTPAxioms(in F, in N)
%%%		F: file
%%%		N: number of axioms in F
%%%		Action: the number of axioms in F is saved

setNumberOfTPTPAxioms(F,N):-
		retractall((numberOfTPTPAxioms(F,_))),
		assertz((numberOfTPTPAxioms(F,N))).

%%% getNumberOfTPTPLines(in F, out N)
%%%		F: file
%%%		N: number of text lines in F

getNumberOfTPTPLines(F,N):-
		numberOfTPTPLines(F,N).

%%% setNumberOfTPTPLines(in F, in N)
%%%		F: file
%%%		N: number of text lines in F
%%%		Action: the number of text lines in F is saved

setNumberOfTPTPLines(F,N):-
		retractall((numberOfTPTPLines(F,_))),
		assertz((numberOfTPTPLines(F,N))).


%%% saveTPTPGoal(in FILE, in FL, in LL, in K, in F)
%%%		FILE: goal file
%%%		FL: first line of the goal in FILE
%%%		LL: last line of the goal in FILE
%%%		K: goal key
%%%		F: goal formula
%%%		Action: the goal is stored

saveTPTPGoal(FILE,FL,LL,K,F):-
		(
		 (
		  retract((tptpGoal(FILE,FL,LL,K,F))),
		  !
		 );
		 (
		  true
		 )
		),
		assertz((tptpGoal(FILE,FL,LL,K,F))).

%%% getTPTPGoal(in K, out F)
%%%		K: goal key
%%%		F: goal formula

getTPTPGoal(K,F):-
		tptpGoal(_,_,_,K,F).

%%% getTPTPGoalFile(in K, out F, out FILE)
%%%		K: goal key
%%%		F: goal formula
%%%		FILE: goal file

getTPTPGoalFile(K,F,FILE):-
		tptpGoal(FILE,_,_,K,F).

%%% setTPTPGoal(in K,in F)
%%%		K: goal key
%%%		F: goal formula
%%%		Action: the goal of key K is update with F

setTPTPGoal(K,F):-
		tptpGoal(FILE,FL,LL,K,_),
		retract((tptpGoal(_,_,_,K,_))),
		assertz((tptpGoal(FILE,FL,LL,K,F))).

%%% getNumberOfTPTPGoals(in F, out N)
%%%		F: file
%%%		N: number of goals in F

getNumberOfTPTPGoals(F,N):-
		numberOfTPTPGoals(F,N).

%%% setNumberOfTPTPGoals(in F, out N)
%%%		F: file
%%%		N: number of goals in F
%%%		Action: the number of goals in F is saved

setNumberOfTPTPGoals(F,N):-
		retractall((numberOfTPTPGoals(F,_))),
		assertz((numberOfTPTPGoals(F,N))).

%%% getNumberOfTPTPTests(out T)
%%%		T: number of tests

getNumberOfTPTPTests(T):-
		numberOfTPTPTests(T).

%%% setNumberOfTPTPTests(in T)
%%%		T: number of tests
%%%		Action: the number of tests is saved

setNumberOfTPTPTests(T):-
		retractall((numberOfTPTPTests(_))),
		assertz((numberOfTPTPTests(T))).

%%% saveAdimenTPTPAxioms

saveAdimenCSRAxioms:-
		open('AdimenCSRAxioms.pl',write,S),
		findall(_,(numberOfTPTPAxioms(SK,_),tptpAxiom(SK,_,_,_,_,AK,T,F),AK \= 'text',term_to_atom(axiom(SK,AK,F,T),A),print_to_file(S,A),print_to_file(S,'.\n')),_),
		close(S).

%%% saveAdimenTPTPTests

saveAdimenCSRTests:-
		open('AdimenCSRTests.pl',write,S),
		findall(_,(tptpTestInformation(K,M,I,TKG1,TKG2),term_to_atom(testInformation(K,M,I,TKG1,TKG2),A),print_to_file(S,A),print_to_file(S,'.\n')),_),
		numberOfTPTPTests(T),
		term_to_atom(numberOfTPTPTests(T),AT),
		print_to_file(S,AT),
		print_to_file(S,'.\n'),
		close(S).

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% %																				% %
% %		List of ontologies:														% %
% %																				% %
% %			We use the (dynamic) predicate "tptpFileList/1"						% %
% %																				% %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

:- dynamic	tptpFileList/1.

:- retractall((tptpFileList(_))).

%%% OpenCyc

%:- assertz((tptpFileList([
%							'CSR002+1-v6.3.0',
%							'CSR002+2-v6.3.0',
%							'CSR002+3-v6.3.0'
%]))).

%%% TPTP-SUMO

%:- assertz((tptpFileList(['CSR003+1-v5.3.0']))).

%%% AIM

:- assertz((tptpFileList(['DolceCASL']))).

%%% getTPTPFileList(out L)
%%%		L: list of TPTP files

getTPTPFileList(L):-
		tptpFileList(L).

%%% setTPTPFileList(in L)
%%%		L: list of TPTP files
%%%		Action: the TPTP file list is updated to L

setTPTPFileList(L):-
		retractall((tptpFileList(_))),
		assertz((tptpFileList(L))).
